-- USER CONFIG AREA -----------------------------------------------------------

console = true -- true/false: display debug messages in the console

replace_name = true -- Replace items names by region names from CSV

delete_items = true -- Delete items outside regions

------------------------------------------------------- END OF USER CONFIG AREA

-- Note: Use it on items that have original edges and rate 1.
-- Note: No CSV validation

-- UTILITIES -------------------------------------------------------------


-- Display a message in the console for debugging
function Msg(value)
  --if console then
    reaper.ShowConsoleMsg(tostring(value) .. "\n")
  --end
end



function tablelength(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end






-- CSV to Table
-- http://lua-users.org/wiki/LuaCsv
function ParseCSVLine (line,sep)
  local res = {}
  local pos = 1
   sep = sep or ','
  while true do
    local c = string.sub(line,pos,pos)
    if (c == "") then break end
    if (c == '"') then
      -- quoted value (ignore separator within)
      local txt = ""
      repeat
        local startp,endp = string.find(line,'^%b""',pos)
        txt = txt..string.sub(line,startp+1,endp-1)
        pos = endp + 1
        c = string.sub(line,pos,pos)
        if (c == '"') then txt = txt..'"' end
        -- check first char AFTER quoted string, if it is another
        -- quoted string without separator, then append it
        -- this is the way to "escape" the quote char in a quote. example:
        -- value1,"blub""blip""boing",value3 will result in blub"blip"boing for the middle
      until (c ~= '"')
      table.insert(res,txt)
      assert(c == sep or c == "")
      pos = pos + 1
    else
      -- no quotes used, just look for the first separator
      local startp,endp = string.find(line,sep,pos)
      if (startp) then
        table.insert(res,string.sub(line,pos,startp-1))
        pos = endp + 1
      else
        -- no separator found -> use rest of string and terminate
        table.insert(res,string.sub(line,pos))
        break
      end
    end
  end
  return res
end
 

function read_lines(filepath)
  
  lines = {}

  f = io.input(filepath)
  repeat

    s = f:read ("*l") -- read one line

    if s then -- if not end of file (EOF)
      table.insert(lines, ParseCSVLine( s, ',' ))
    end

  until not s -- until end of file

  f:close()


  return lines --'lines' is a table of values of table 'line' so like a excel array of csv values
  
end


--------------------------------------------------------- END OF UTILITIES


function get_file_name(file)

  return string.match(file, "/[^/]*$")
  
end

function GetFileExtension(ext)
  return ext:match("^.+(%..+)$")
end

function stretch_item_to_pos(xpos)
  
  ---Script: mpl_Stretch selected items to edit cursor preserve snapoffset.eel
  
  item = reaper.GetSelectedMediaItem(0, 0)
  take = reaper.GetActiveTake(item)
  rate = reaper.GetMediaItemTakeInfo_Value(take, "D_PLAYRATE")
  pos = reaper.GetMediaItemInfo_Value(item, "D_POSITION")
  length = reaper.GetMediaItemInfo_Value(item, "D_LENGTH")
  snapoffset = reaper.GetMediaItemInfo_Value(item, "D_SNAPOFFSET")
  
  invsnapoffset = length - snapoffset
  snapoffset_pos = pos + snapoffset
  item_end = pos + length
  editcur = xpos --reaper.GetCursorPosition()
  abs_beetween = math.abs(snapoffset_pos - editcur)
  itemcenter = pos + length/2
  k = 1
  
  if snapoffset ~= 0 then
    if editcur < snapoffset_pos then
      k = abs_beetween / snapoffset
      newlength = item_end - editcur
      newpos = editcur
      newsnapoffset = snapoffset * k 
      snapoffset0 = snapoffset * rate
      reaper.SetTakeStretchMarker(take, -1, 0)
      reaper.SetTakeStretchMarker(take, -1, snapoffset0)
    end  
  
    if editcur > snapoffset_pos then
      k = abs_beetween / invsnapoffset
      newlength = editcur - pos
      newpos = pos
      newsnapoffset = snapoffset
      snapoffset0 = snapoffset * rate
      length0 = length * rate
      reaper.SetTakeStretchMarker(take, -1, snapoffset0)
      reaper.SetTakeStretchMarker(take, -1, length0)
    end
    if editcur == snapoffset_pos then
      newlength = length
      newpos = pos
      newsnapoffset = snapoffset
    end
  reaper.SetMediaItemInfo_Value(item, "D_SNAPOFFSET", newsnapoffset)    
  end
  
  if snapoffset == 0 then
    if editcur < itemcenter then
      fut_lenght = item_end - editcur
      k = fut_lenght / length
      newlength = length * k
      newpos = editcur
    end
    if editcur > itemcenter then
      fut_length = editcur - pos
      k = fut_length / length
      newlength = length * k
      newpos = editcur - newlength
    end
    if editcur == itemcenter then
      newlength = length
      newpos = pos
    end
    newrate = rate / k 
    reaper.SetMediaItemTakeInfo_Value(take, "D_PLAYRATE", newrate)  
  end
     
  reaper.SetMediaItemInfo_Value(item, "D_POSITION", newpos)
  reaper.SetMediaItemInfo_Value(item, "D_LENGTH", newlength)
   
  
  if snapoffset ~= 0 then
    if editcur < snapoffset_pos then
      newsnapoffset0 = (snapoffset_pos - newpos) * rate
      reaper.SetTakeStretchMarker(take, 1, newsnapoffset0)
    end
    if editcur > snapoffset_pos then
      newlength0 = (editcur - newpos) * rate
      reaper.SetTakeStretchMarker(take, 0, snapoffset0)
      reaper.SetTakeStretchMarker(take, 1, newlength0)
    end
  end

end


function choose_media()
  
  no_track_selection = 0
  -- Move cursor to time selection
  start_time, end_time = reaper.GetSet_LoopTimeRange2(0, false, false, 0, 0, false)
  if start_time == end_time then
    reaper.MB( "Set Time Selection", "Select Track & Time Selection", 0 )
    no_track_selection = 1 goto skip
  
  end
  
  get_track =  reaper.CountSelectedTracks2( 0, 0 )
  if get_track ~= 1 then
    reaper.MB( "Select One Target Track", "Select Track & Time Selection", 0 )
    no_track_selection = 1 goto skip
  
  end
  
  retval, item_source = reaper.GetUserFileNameForRead("", "Choose Instrument/Drums Media File", "")
  --Msg(item_source)
  item_name = string.match(item_source, "[^\\]*$") --WIN get_file_name(item_source)
  
  master_tempo = reaper.Master_GetTempo() -- Note: Not current
  --Msg(item_name)
  org_bpm = tonumber(item_name:match('(%d+)bpm')) or 120
  numerator, denominator = item_name:match('(%d+)-(%d+)') --beats and unit from file name
  csv_filename = item_source:gsub("%.[^%.]+$", '.csv') --Change extension to csv disregard any "." within the filename
  file_ext = GetFileExtension(item_name)
  --Msg("File ext "..file_ext)
  if file_ext == ".MID" or file_ext == ".mid" or file_ext ==".midi" then
    btn_reatrak_mid()
  else
    btn_reatrak()
  end  
  ::skip::
  
end


function btn_reatrak()
  
  reaper.PreventUIRefresh( 1 )
  
  if no_track_selection == 1 then goto skip2 end
  
  read_lines(csv_filename)
  
  chord_items_in_media = tablelength(lines)
  
  reaper.SetEditCurPos( start_time, 0, 0 )
  
  reaper.Main_OnCommand(40289, 0) -- Item: Unselect all items
  
  reaper.Main_OnCommand(40718, 0) -- Item: Select all items on selected tracks in current time selection
  
  reaper.Main_OnCommand(40006, 0) -- Item: Remove items

  reaper.Main_OnCommand(40754, 0) -- Enable snap
  
  reaper.SetEditCurPos2( 0, start_time, 0, 0 )
  
  reaper.InsertMedia( item_source, 0 )
   
  inserted_item = reaper.GetSelectedMediaItem( 0, 0 )
  
  inserted_take = reaper.GetMediaItemTake( inserted_item, 0)
  
  reaper.Main_OnCommand(40699, 0) -- Edit: Cut items
  
  pos = 0
  
  region_count , num_markersOut, num_regionsOut = reaper.CountProjectMarkers(0)
  
  for i=0, region_count -1 do
  
    
    retval, isrgnOut, posOut, rgnendOut, region_name, markrgnindexnumberOut, colorOut = reaper.EnumProjectMarkers3(0, i)      
    
    retval, posOut_measures, posOut_cml, posOut_fullbeats, posOut_cdenom = reaper.TimeMap2_timeToBeats( 0, posOut )
    
    retval, rgnendOut_measures, rgnendOut_cml, rgnendOut_fullbeats, rgnendOut_cdenom = reaper.TimeMap2_timeToBeats( 0, rgnendOut )
    
    retval, start_time_measures, start_time_cml, start_time_fullbeats, start_time_cdenom = reaper.TimeMap2_timeToBeats( 0, start_time )
    
    retval, end_time_measures, end_time_cml, end_time_fullbeats, end_time_cdenom = reaper.TimeMap2_timeToBeats( 0, end_time )
    
    
    region_length = rgnendOut - posOut
    region_length_beats = rgnendOut_fullbeats - posOut_fullbeats
    
    --if posOut_fullbeats >= start_time_fullbeats and posOut_fullbeats < end_time_fullbeats then 
    if posOut >= start_time - 0.005 and posOut < end_time - 0.005 then
    
      for c = 1, chord_items_in_media -1 do
      
        if region_name == lines[c+1][2] then
        
          reaper.SetEditCurPos2( 0, start_time, 0, 0 )
          
          
          --reaper.InsertMedia( item_source, 0 )
          reaper.Main_OnCommand(40058, 0) -- Item: Paste items/tracks (old-style handling of hidden tracks)
           
          inserted_item = reaper.GetSelectedMediaItem( 0, 0 )
          
          inserted_take = reaper.GetMediaItemTake( inserted_item, 0)
          
          reaper.Main_OnCommand(40630, 0) -- Go to start of time selection
          
          
          
          start_in_media = lines[c+1][3]
          end_in_media = lines[c+1][4]
          length_in_media = lines[c+1][5]
          
          pos = 0
          
          
          startbars, startbeats, startcbeats = start_in_media:match("(%d+).(%d+).(%d+)")
          endbars, endbeats, endcbeats = end_in_media:match("(%d+).(%d+).(%d+)")
          lengthbars, lengthbeats, lengthcbeats = length_in_media:match("(%d+).(%d+).(%d+)")
          
          time_pos_start = reaper.TimeMap2_beatsToTime( 0, startbeats, startbars )
          
          time_pos_end = reaper.TimeMap2_beatsToTime( 0, endbeats, endbars )
          
          time_length = reaper.TimeMap2_beatsToTime( 0, lengthbeats, lengthbars )
          
          start_beat_time = ((((startbars-1) * numerator) + startbeats -1) * 60) / org_bpm   
          
          end_beat_time = ((((endbars-1) * numerator) + endbeats -1)  * 60) / org_bpm 
          
          length_beat_time = ((((lengthbars-1) * numerator) + lengthbeats -1)  * 60) / org_bpm
          
          length_beat = (((lengthbars) * numerator) + lengthbeats )
          
          
          reaper.SetMediaItemInfo_Value(inserted_item, "D_POSITION", posOut)
          
          retval, a_take_name = reaper.GetSetMediaItemTakeInfo_String(inserted_take, "P_NAME", lines[c+1][2], 1)
          
          source_start = reaper.SetMediaItemTakeInfo_Value( inserted_take, "D_STARTOFFS", start_beat_time )
          
         
          item_length = end_beat_time - start_beat_time -- ITEM @ ORG BPM
          
          item_length_region = time_pos_end - time_pos_start -- ITEM @ CURRENT BPM
          
          
          if length_beat > region_length_beats then
            
            item_length = (region_length_beats * 60) / org_bpm --reaper.TimeMap2_beatsToTime( 0, region_length_beats)
            
            reaper.SetMediaItemInfo_Value( inserted_item, "D_LENGTH", item_length)
            
            time_pos = posOut + region_length
            stretch_item_to_pos(time_pos)
            
            commandID1 = reaper.NamedCommandLookup("_SWS_AWITEMTBASEBEATSTRETCH")
            reaper.Main_OnCommand(commandID1, 0) -- SWS/AW: Set selected items timebase to beats (auto-stretch at tempo changes)
            
            reaper.Main_OnCommand(41193, 0) -- Item: Remove fade in and fade out
            
            reaper.Main_OnCommand(40416, 0) -- Item navigation: Select and move to previous item
            
            reaper.Main_OnCommand(41193, 0) -- Item: Remove fade in and fade out
            
          else  
            reaper.SetMediaItemInfo_Value( inserted_item, "D_LENGTH", item_length)
            
            item_end_beats = posOut_fullbeats + length_beat
           
            time_pos = reaper.TimeMap2_beatsToTime( 0, item_end_beats)
            
            stretch_item_to_pos(time_pos)
            
            commandID1 = reaper.NamedCommandLookup("_SWS_AWITEMTBASEBEATSTRETCH")
            reaper.Main_OnCommand(commandID1, 0) -- SWS/AW: Set selected items timebase to beats (auto-stretch at tempo changes)
            
            reaper.Main_OnCommand(41193, 0) -- Item: Remove fade in and fade out
            
            reaper.Main_OnCommand(40416, 0) -- Item navigation: Select and move to previous item
            
            reaper.Main_OnCommand(41193, 0) -- Item: Remove fade in and fade out
            
          end  
          
           
          reaper.Main_OnCommand(40289, 0) -- Item: Unselect all items
           
          
          pos = time_pos
          
         
          
          while time_pos < rgnendOut do -- Repeat Item Chord
            
            --reaper.InsertMedia( item_source, 0) -- 8)
            reaper.Main_OnCommand(40058, 0) -- Item: Paste items/tracks (old-style handling of hidden tracks)
            
            reaper.SetEditCurPos2( 0, pos, 0, 0 )
                        
            inserted_item = reaper.GetSelectedMediaItem( 0, 0 )
            
            inserted_take = reaper.GetMediaItemTake( inserted_item, 0)
            
            reaper.SetMediaItemInfo_Value(inserted_item, "D_POSITION", pos)
            
            retval, a_take_name = reaper.GetSetMediaItemTakeInfo_String(inserted_take, "P_NAME", lines[c+1][2], 1)
            
            source_start = reaper.SetMediaItemTakeInfo_Value( inserted_take, "D_STARTOFFS", start_beat_time )
            
             
            if pos + item_length > rgnendOut then
            
              retval, post_measures, pos_cml, pos_fullbeats, pos_cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
            
              item_length = ((rgnendOut_fullbeats - pos_fullbeats) * 60) / org_bpm
              
              reaper.SetMediaItemInfo_Value( inserted_item, "D_LENGTH", item_length)
              pos = rgnendOut
              stretch_item_to_pos(pos)
              
              commandID1 = reaper.NamedCommandLookup("_SWS_AWITEMTBASEBEATSTRETCH")
              reaper.Main_OnCommand(commandID1, 0) -- SWS/AW: Set selected items timebase to beats (auto-stretch at tempo changes)
              
              reaper.Main_OnCommand(41193, 0) -- Item: Remove fade in and fade out
              
              reaper.Main_OnCommand(40289, 0) -- Item: Unselect all items
              
              reaper.Main_OnCommand(40416, 0) -- Item navigation: Select and move to previous item
              
              reaper.Main_OnCommand(41193, 0) -- Item: Remove fade in and fade out
              break
              
            else  
            
              retval, post_measures, pos_cml, pos_fullbeats, pos_cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
              
              reaper.SetMediaItemInfo_Value( inserted_item, "D_LENGTH", item_length)
              
              item_end_beats = pos_fullbeats + length_beat
              
              time_pos = reaper.TimeMap2_beatsToTime( 0, item_end_beats)
             
              stretch_item_to_pos(time_pos)
              
              commandID1 = reaper.NamedCommandLookup("_SWS_AWITEMTBASEBEATSTRETCH")
              reaper.Main_OnCommand(commandID1, 0) -- SWS/AW: Set selected items timebase to beats (auto-stretch at tempo changes)
              
              reaper.Main_OnCommand(41193, 0) -- Item: Remove fade in and fade out
              
              reaper.Main_OnCommand(40416, 0) -- Item navigation: Select and move to previous item
              
              reaper.Main_OnCommand(41193, 0) -- Item: Remove fade in and fade out
               
            end 
            
          
            reaper.Main_OnCommand(40289, 0) -- Item: Unselect all items
            reaper.SetEditCurPos2( 0, pos, 0, 0 )
            
            
            
            pos = time_pos
            
            
          end
          
        end
        
      end
      
    end
    

    ::skip_end::
  end
  
  ::skip2::
   
  reaper.Main_OnCommand(40289, 0) -- Item: Unselect all items
  reaper.UpdateArrange()
  reaper.PreventUIRefresh( -1 )
  
end


function main()
  item = reaper.GetSelectedMediaItem( 0, 0 )
  if not item then return end
  take = reaper.GetActiveTake( item )
  if not take or not  reaper.TakeIsMIDI( take ) then return end
  
  ppqpos = reaper.MIDI_GetPPQPosFromProjQN( take, 32 ) -- get PPQ position ticks for 32 beats
  newpos =  reaper.MIDI_GetProjTimeFromPPQPos( take, ppqpos ) -- convert to seconds 
  reaper.SetMediaItemInfo_Value( item, 'D_POSITION', newpos) -- apply
  reaper.SetMediaItemTakeInfo_Value( take, 'D_STARTOFFS', newpos )
end



function btn_reatrak_mid()

  reaper.PreventUIRefresh( 1 )
  
  if no_track_selection == 1 then goto skip2 end
  
  read_lines(csv_filename)
  
  chord_items_in_media = tablelength(lines)
  
  reaper.SetEditCurPos( start_time, 0, 0 )
  
  reaper.Main_OnCommand(40289, 0) -- Item: Unselect all items
  
  reaper.Main_OnCommand(40718, 0) -- Item: Select all items on selected tracks in current time selection
  
  reaper.Main_OnCommand(40006, 0) -- Item: Remove items

  reaper.Main_OnCommand(40754, 0) -- Enable snap
  
  reaper.SetEditCurPos2( 0, start_time, 0, 0 )
  
  reaper.InsertMedia( item_source, 0 )
   
  inserted_item = reaper.GetSelectedMediaItem( 0, 0 )
  
  inserted_take = reaper.GetMediaItemTake( inserted_item, 0)
  
  reaper.Main_OnCommand(40699, 0) -- Edit: Cut items
  
  pos = 0
  
  region_count , num_markersOut, num_regionsOut = reaper.CountProjectMarkers(0)
  
  for i=0, region_count -1 do
  
    
    retval, isrgnOut, posOut, rgnendOut, region_name, markrgnindexnumberOut, colorOut = reaper.EnumProjectMarkers3(0, i)      
    
    retval, posOut_measures, posOut_cml, posOut_fullbeats, posOut_cdenom = reaper.TimeMap2_timeToBeats( 0, posOut )
    
    retval, rgnendOut_measures, rgnendOut_cml, rgnendOut_fullbeats, rgnendOut_cdenom = reaper.TimeMap2_timeToBeats( 0, rgnendOut )
    
    retval, start_time_measures, start_time_cml, start_time_fullbeats, start_time_cdenom = reaper.TimeMap2_timeToBeats( 0, start_time )
    
    retval, end_time_measures, end_time_cml, end_time_fullbeats, end_time_cdenom = reaper.TimeMap2_timeToBeats( 0, end_time )
    
    
    region_length = rgnendOut - posOut
    region_length_beats = rgnendOut_fullbeats - posOut_fullbeats
    
    --if posOut_fullbeats >= start_time_fullbeats and posOut_fullbeats < end_time_fullbeats then 
    if posOut >= start_time - 0.005 and posOut < end_time - 0.005 then
    
      for c = 1, chord_items_in_media -1 do
      
        if region_name == lines[c+1][2] then
        
          reaper.SetEditCurPos2( 0, start_time, 0, 0 )
          
          
          --reaper.InsertMedia( item_source, 0 )
          reaper.Main_OnCommand(40058, 0) -- Item: Paste items/tracks (old-style handling of hidden tracks)
           
          inserted_item = reaper.GetSelectedMediaItem( 0, 0 )
          
          inserted_take = reaper.GetMediaItemTake( inserted_item, 0)
          
          reaper.Main_OnCommand(40630, 0) -- Go to start of time selection
          
          
          
          start_in_media = lines[c+1][3]
          end_in_media = lines[c+1][4]
          length_in_media = lines[c+1][5]
          
          pos = 0
          
          
          startbars, startbeats, startcbeats = start_in_media:match("(%d+).(%d+).(%d+)")
          endbars, endbeats, endcbeats = end_in_media:match("(%d+).(%d+).(%d+)")
          lengthbars, lengthbeats, lengthcbeats = length_in_media:match("(%d+).(%d+).(%d+)")
          
          time_pos_start = reaper.TimeMap2_beatsToTime( 0, startbeats, startbars )
          
          time_pos_end = reaper.TimeMap2_beatsToTime( 0, endbeats, endbars )
          
          time_length = reaper.TimeMap2_beatsToTime( 0, lengthbeats, lengthbars )
          
          start_beat_time = ((((startbars-1) * numerator) + startbeats -1) * 60) / org_bpm   
          
          end_beat_time = ((((endbars-1) * numerator) + endbeats -1)  * 60) / org_bpm 
          
          length_beat_time = ((((lengthbars-1) * numerator) + lengthbeats -1)  * 60) / org_bpm
          
          length_beat = (((lengthbars) * numerator) + lengthbeats )
          
          
          reaper.SetMediaItemInfo_Value(inserted_item, "D_POSITION", posOut)
          
          retval, a_take_name = reaper.GetSetMediaItemTakeInfo_String(inserted_take, "P_NAME", lines[c+1][2], 1)
          
          source_start = reaper.SetMediaItemTakeInfo_Value( inserted_take, "D_STARTOFFS", start_beat_time )
          
         
          item_length = end_beat_time - start_beat_time -- ITEM @ ORG BPM
          
          item_length_region = time_pos_end - time_pos_start -- ITEM @ CURRENT BPM
          
          
          if length_beat > region_length_beats then
            
            item_length = (region_length_beats * 60) / org_bpm --reaper.TimeMap2_beatsToTime( 0, region_length_beats)
            
            reaper.SetMediaItemInfo_Value( inserted_item, "D_LENGTH", item_length)
            
            time_pos = posOut + region_length
            --stretch_item_to_pos(time_pos)
            
            commandID1 = reaper.NamedCommandLookup("_SWS_AWITEMTBASEBEATSTRETCH")
            reaper.Main_OnCommand(commandID1, 0) -- SWS/AW: Set selected items timebase to beats (auto-stretch at tempo changes)

            
          else  
            reaper.SetMediaItemInfo_Value( inserted_item, "D_LENGTH", item_length)
            
            item_end_beats = posOut_fullbeats + length_beat
           
            time_pos = reaper.TimeMap2_beatsToTime( 0, item_end_beats)
            
            stretch_item_to_pos(time_pos)
            
            commandID1 = reaper.NamedCommandLookup("_SWS_AWITEMTBASEBEATSTRETCH")
            reaper.Main_OnCommand(commandID1, 0) -- SWS/AW: Set selected items timebase to beats (auto-stretch at tempo changes)
            

          end  
          
           
          reaper.Main_OnCommand(40289, 0) -- Item: Unselect all items
           
          
          pos = time_pos
          
         
          
          while time_pos < rgnendOut do -- Repeat Item Chord
            
            --reaper.InsertMedia( item_source, 0) -- 8)
            reaper.Main_OnCommand(40058, 0) -- Item: Paste items/tracks (old-style handling of hidden tracks)
            
            reaper.SetEditCurPos2( 0, pos, 0, 0 )
                        
            inserted_item = reaper.GetSelectedMediaItem( 0, 0 )
            
            inserted_take = reaper.GetMediaItemTake( inserted_item, 0)
            
            reaper.SetMediaItemInfo_Value(inserted_item, "D_POSITION", pos)
            
            retval, a_take_name = reaper.GetSetMediaItemTakeInfo_String(inserted_take, "P_NAME", lines[c+1][2], 1)
            
            source_start = reaper.SetMediaItemTakeInfo_Value( inserted_take, "D_STARTOFFS", start_beat_time )
            
             
            if pos + item_length > rgnendOut then
            
              retval, post_measures, pos_cml, pos_fullbeats, pos_cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
            
              item_length = ((rgnendOut_fullbeats - pos_fullbeats) * 60) / org_bpm
              
              reaper.SetMediaItemInfo_Value( inserted_item, "D_LENGTH", item_length)
              pos = rgnendOut
              stretch_item_to_pos(pos)
              
              commandID1 = reaper.NamedCommandLookup("_SWS_AWITEMTBASEBEATSTRETCH")
              reaper.Main_OnCommand(commandID1, 0) -- SWS/AW: Set selected items timebase to beats (auto-stretch at tempo changes)
              

              break
              
            else  
            
              retval, post_measures, pos_cml, pos_fullbeats, pos_cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
              
              reaper.SetMediaItemInfo_Value( inserted_item, "D_LENGTH", item_length)
              
              item_end_beats = pos_fullbeats + length_beat
              
              time_pos = reaper.TimeMap2_beatsToTime( 0, item_end_beats)
             
              stretch_item_to_pos(time_pos)
              
              commandID1 = reaper.NamedCommandLookup("_SWS_AWITEMTBASEBEATSTRETCH")
              reaper.Main_OnCommand(commandID1, 0) -- SWS/AW: Set selected items timebase to beats (auto-stretch at tempo changes)
              

               
            end 
            
          
            reaper.Main_OnCommand(40289, 0) -- Item: Unselect all items
            reaper.SetEditCurPos2( 0, pos, 0, 0 )
            
            
            
            pos = time_pos
            
            
          end
          
        end
        
      end
      
    end
    

    ::skip_end::
  end
  
  ::skip2::
   
  reaper.Main_OnCommand(40289, 0) -- Item: Unselect all items
  reaper.UpdateArrange()
  reaper.PreventUIRefresh( -1 )
  
end




  choose_media()
 -- btn_reatrak()
  
  
  reaper.Main_OnCommand(40630, 0) -- Go to start of time selection
  
  reaper.UpdateArrange()


